class StopuhrTimer1
{
    public:
        StopuhrTimer1(RS232* rs232,int vt = 1024,int pulse_pro_umdrehung = 32, double radradius_in_metern = 0.014)
	    {
		    PI = 3.141592653589793;
            vorteilung = vt;
		    pulseproumdrehung = pulse_pro_umdrehung;
			radradiusinmetern = radradius_in_metern;

            if(vorteilung!=1 && vorteilung!=8 && vorteilung !=64 && vorteilung!=256 && vorteilung!=1024)
			{
			    rs232->sendeText("\n\rUngueltige Vorteilung in StopuhrTimer1. Verwende 1024\n\r");
				vorteilung = 1024;
			}

		    rs232->sendeText("\n\rInstanz der Klasse *StopuhrTimer1* gebildet\n\r");
		    rs232->sendeText("\n\r***** Stopuhr Timer 1 *****\n\r");
		    rs232->sendeText("AVR-Konfiguration fuer ATmega32:\n\r");
		    rs232->sendeText("Betroffene Pins: 20 (PD6/ICP) als Capture-Eingang \n\r");
		    rs232->sendeText("Betroffene Register: TCCR1A TCCR1B\n\r");
		    rs232->sendeText("Bezeichnung: Timer1 auf 16 Zaehlmodus setzen.\n\r");
		    rs232->sendeText("Aktuelle Vorteilung: ");
		    rs232->sendeIntegerZahl(vorteilung);
		    rs232->sendeText("\n\rMoegliche Vorteilungen: 1, 8, 64, 256, 1024\n\r");
		    rs232->sendeText("Methoden:\n\r");
		    rs232->sendeText("resetZaehler() \n\r");
		    rs232->sendeText("unsigned int getZaehlstand() \n\r");
		    rs232->sendeText("bool pruefeUeberlauf() \n\r");
	    }

		StopuhrTimer1(int vt = 1024,int pulse_pro_umdrehung = 32, double radradius_in_metern = 0.014)
		{
		    vorteilung = vt;
		}

        //Dauer der bergebenen Timerschritte in Sekunden
        double berechneDauerSek(unsigned int pulse)
		{
		    return ((double)vorteilung*(double)pulse)/TAKT;
		}

        double berechneWinkelgeschw(unsigned int pulse)
		{
		    //32 Pulse pro Umdrehung => zwischen zwei Pulsen werden (2*PI)/32 rad durchlaufen
			//rad / sek = ((2*PI)/32)/gestopteZeit = 0.19634954084936207740391521145497/berechneDauerSek
		    return (2.0*PI*TAKT)/((double)pulseproumdrehung*(double)vorteilung*(double)pulse);
		}

        //Umdrehungen pro Minute
        double berechneUPM(unsigned int pulse)
		{
		    //rad/sek *60 /2PI = 60/(berechneDauerSek*32)
			
		    return (60.0*TAKT)/((double)vorteilung*(double)pulse*(double)pulseproumdrehung);
		}

        //Geschwindigkeit in m/s
        double berechneGeschwindigkeit(unsigned int pulse)
		{
		    return (2.0*PI*radradiusinmetern*TAKT)/((double)vorteilung*(double)pulse*(double)pulseproumdrehung);
		}

        void resetZaehler()
		{
            /* Save global interrupt flag */
            akku = SREG;
            /* Disable interrupts */
            cli();
            /* Set TCNT1 to i */
            TCNT1 = 0;
            /* Restore global interrupt flag */
            SREG = akku;
		}

        unsigned int getZaehlstand()
		{
		    return ICR1;
		}

        bool pruefeUeberlauf()
		{
		    //prft, ob es einen Zaehlerueberlauf gab,
			//wenn ja, wird true zurckgegeben, sonst false:

            akku = TIFR;

			if( (akku & (1<<TOV1) ) == 0 )
			{
			    //nein
				return false;
			}
			else
			{
			    //ja, Flag zurcksetzen
                TIFR = akku; //so ?????
				return true;
			}
		}

        void start()
		{
		    //TCCR1A = (0<<COM1A1) | (0<<COM1A0) | (0<<COM1B1) | (0<<COM1B0) | (0<<FOC1A) | (0<<FOC1B) | (0<<PWM11) | (0<<PWM10);
			switch(vorteilung)
			{
			    case 1:
			        TCCR1B = (1<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (0<<CS12) | (0<<CS11) | (1<<CS10);
                break;
			    case 8:
			        TCCR1B = (1<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (0<<CS12) | (1<<CS11) | (0<<CS10);
                break;
			    case 64:
			        TCCR1B = (1<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (0<<CS12) | (1<<CS11) | (1<<CS10);
                break;
			    case 256:
			        TCCR1B = (1<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (1<<CS12) | (0<<CS11) | (0<<CS10);
                break;
			    case 1024:
			        TCCR1B = (1<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (1<<CS12) | (0<<CS11) | (1<<CS10);
                break;
				default:
			        TCCR1B = (1<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (1<<CS12) | (0<<CS11) | (1<<CS10);
				break;
            }
			TIMSK |= (1<<ICF1);
			sei(); //Interrupts frei
			//PD6 ist Eingang:
			DDRD &= 255 - (1<<PD6);
		}

        void stop()
		{
	        TCCR1B = (0<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (0<<CS12) | (0<<CS11) | (0<<CS10);
			TIMSK &= 255 - (1<<ICF1);
			cli();
		}

    private:
        unsigned char akku;
		unsigned int  i;
        int vorteilung;
		int pulseproumdrehung;
	    double radradiusinmetern;
		double PI;
};
